/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2003, 2006                               */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/* d14416    20060515   OTEST1 option, might fail if volume is not   */
/*                       mounted                                     */
/*                                                                   */
/*                                                                   */
/*********************************************************************/
/*********************************************************************
 ** FUNCTION NAME:    QTPA
 ** LOAD MODULE NAME: QTP9
 ** PURPOSE:          PERFORM FUNCTIONS FOR THE TAPE DRIVER
 ** FUNCTION TYPE:    TPF
 ** DOCUMENT NAME:    TAPE31 UGUIDE ON TDRIVERS DISK
 ** LEVEL:            VERSION 1  MODIFICATION 0
 **
 ** INPUT:            ALL INPUT CONTAINED IN ECB MAPPED BY C$TA0EQ
 **
 ** OUTPUT:           NONE
 **
 ** DESCRIPTION:      THIS SEGMENT PERFORMS TESTS ON THE FOLLOWING
 **                   APIS: tape_cntl, tsync, tdspc_q
 **
 *********************************************************************
 ** DEPENDENCIES:     QXTG MAY REQUIRE MODIFICATION IF THIS PROGRAM
 **                   CHANGES HOW IT IS CALLED.
 **
 **                   SINCE THIS SEGMENT IS PART OF A DLM, IF AN
 **                   EXTERNAL CALL IS ADDED FOR WHICH A STUB DOES
 **                   NOT EXIST, A STUB WILL NEED TO BE CREATED.
 **
 ** RESTRICTIONS:     NONE
 **
 ** RELATED SEGMENTS: QXTG         MESSAGE PARSER
 **
 *********************************************************************
 ** EXTERNAL REFERENCES:    NONE
 **
 ** HEADER FILES #INCLUDED: NONE
 **
 ** ACRONYMS: API       APPLICATION PROGRAMMING INTERFACE
 **           TPFTAPE   TPF TAPE FUNCTIONS
 **           C$TA0EQ   TAPE DRIVER MAPPING OF IT'S USE OF GLOBAL AREA
 **
 *********************************************************************
 ** CHANGE ACTIVITY:
 **
 **    UPDATE BY           PLOG #   DATE    DESCRIPTION
 **    ------------------  -------  ------  -------------------------
 **
 *********************************************************************/
#include <c_ta0eq64.h>
#include <tpf/tpfio.h>
#include <tpf/c_eb0eb.h>
#include <tpf/c_dctist.h>
#include <tpf/c_pi1dt.h>

void tape_test();
void tape_test_noblk(char *);
void tape_test_blk(char *);

/**********************************************************************
* QTPA                                                                *
**********************************************************************/
int QTPA()  //gcc
{
  tape_test();
  exit(0);
}

/**********************************************************************
* tape_test_noblk                                                     *
* Runs a series of 10 tests on non-blocked tapes, testing the tape    *
* apis: tape_cntl, tsync, and tdspc_q                                 *
**********************************************************************/
void tape_test_noblk(char *tape_name)
{
  char *data_area = NULL;
  int i;
  struct tpqstat *status;

#define L0SIZE 127

  /*******************************************************************/
  /* Test Case 1:  Open the tape                                     */
  /*******************************************************************/
  /* open the tape for r/w                                           */
  tape_open(tape_name, OUTPUT);

  /*******************************************************************/
  /* Test Case 2:  Initialize the tape                               */
  /*******************************************************************/
  /* make sure the tape is rewound                                   */
  if (tape_cntl(tape_name, CNTL_REW, NO_FALLBACK))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 2 Failed - Could not rewind tape");
    exit(0);
  }

  /* write the sample data to tape                                   */
  for (i = 'a'; i <= 'g'; i++)
  {
    data_area = getcc(D1,
                      GETCC_TYPE+GETCC_FILL,
                      L0,
                      (char) i);

    tape_write(tape_name, D1);
  }

  /* flush the tape buffer                                           */
  tape_cntl(tape_name, CNTL_FLUSH);

  /*******************************************************************/
  /* Test Case 3:  Check the first record                            */
  /*******************************************************************/
  /* rewind the tape                                                 */
  if (tape_cntl(tape_name, CNTL_REW, NO_FALLBACK))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 3 Failed - Could not rewind tape");
    exit(0);
  }

  /* read the first record                                           */
  if ((data_area = tape_read(tape_name, D1, L0)) == NULL)
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 3 Failed - Could not read from tape");
    exit(0);
  }

  /* check a few spots                                               */
  if ((data_area[0] != 'a') || (data_area[L0SIZE-1] != 'a'))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 3 Failed - Data did not match original");
    exit(0);
  }

  relcc(D1);

  /*******************************************************************/
  /* Test Case 4:  Move forward one record                           */
  /*******************************************************************/
  /* move forward one record (to 'c')                                */
  if (tape_cntl(tape_name, CNTL_FSB, D1, 1))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 4 Failed - Could not seek tape forward");
    exit(0);
  }

  /* read the record                                                 */
  if (!(data_area = tape_read(tape_name, D1, L0)))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 4 Failed - Could not read from tape");
    exit(0);
  }

  /* check a few spots                                               */
  if ((data_area[0] != 'c') || (data_area[L0SIZE-1] != 'c'))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 4 Failed - Data did not match original");
    exit(0);
  }

  relcc(D1);

  /*******************************************************************/
  /* Test Case 5:  Seek tape back 1 record                           */
  /*******************************************************************/
  /* move backward one record (to 'c')                               */
  if (tape_cntl(tape_name, CNTL_BSB, D1, 1))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 5 Failed - Could not seek tape backward");
    exit(0);
  }

  /* read the record                                                 */
  if (!(data_area = tape_read(tape_name, D1, L0)))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 5 Failed - Could not read from tape");
    exit(0);
  }

  /* check a few spots                                               */
  if ((data_area[0] != 'c') || (data_area[L0SIZE-1] != 'c'))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 5 Failed - Data did not match original");
    exit(0);
  }

  relcc(D1);

  /*******************************************************************/
  /* Test Case 6:  Seek forward 3 records                            */
  /*******************************************************************/
  /* forward 3 records ('g')                                         */
  if (tape_cntl(tape_name, CNTL_FSB, D1, 3))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 6 Failed - Could not seek tape forward");
    exit(0);
  }

  /* read the record                                                 */
  if (!(data_area = tape_read(tape_name, D1, L0)))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 6 Failed - Could not read from tape");
    exit(0);
  }

  /* check a few spots                                               */
  if ((data_area[0] != 'g') || (data_area[L0SIZE-1] != 'g'))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 6 Failed - Data did not match original");
    exit(0);
  }

  relcc(D1);

  /*******************************************************************/
  /* Test Case 7:  Seek backward 5 records                           */
  /*******************************************************************/
  /* backward 5 records ('c')                                        */
  if (tape_cntl(tape_name, CNTL_BSB, D1, 5))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 7 Failed - Could not seek tape backward");
    exit(0);
  }

  /* read the record                                                 */
  if (!(data_area = tape_read(tape_name, D1, L0)))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 7 Failed - Could not read from tape");
    exit(0);
  }

  /* check a few spots                                               */
  if ((data_area[0] != 'c') || (data_area[L0SIZE-1] != 'c'))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 7 Failed - Data did not match original");
    exit(0);
  }

  relcc(D1);

  /*******************************************************************/
  /* Test Case 8:  Try to seek past end of tape                      */
  /*******************************************************************/
  /* rewind tape                                                     */
  if (tape_cntl(tape_name, CNTL_REW, NO_FALLBACK))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 8 Failed - Could not rewind tape");
    exit(0);
  }

  /* write one record                                                */
  data_area = getcc(D1,
                    GETCC_TYPE+GETCC_FILL,
                    L1,
                    'z');

  tape_write(tape_name, D1);
  tape_cntl(tape_name, CNTL_FLUSH);

  /* read past end of tape                                           */
  if ((data_area = tape_read(tape_name, D1, L1)))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 8 Failed - Read past end of tape");
    exit(0);
  }

  /* seek past end of tape                                           */
  if (!tape_cntl(tape_name, CNTL_FSB, D1, 2))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 8 Failed - Seeked past end of tape");
    exit(0);
  }

  /*******************************************************************/
  /* Test Case 9:  Seek past beginning of tape                       */
  /*******************************************************************/
  /* rewind                                                          */
  if (tape_cntl(tape_name, CNTL_REW, NO_FALLBACK))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 9 Failed - Could not rewind tape");
    exit(0);
  }

  /* seek backward                                                   */
  if (!tape_cntl(tape_name, CNTL_BSB, D1, 2))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "TEST1 Test 9 Failed - Seeked past beginning of tape");
    exit(0);
  }

  /*******************************************************************/
  /* Test Case 10: Tape queue length and syncing                     */
  /*******************************************************************/
  /* rewind tape                                                     */
  if (tape_cntl(tape_name, CNTL_REW, NO_FALLBACK))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 10 Failed - Could not rewind tape");
    exit(0);
  }

  /* grab tape                                                       */
  tasnc(tape_name);

  /* write lots of records                                           */
  for (i = 0; i < 50; i++)
  {
    data_area = getcc(D1,
                      GETCC_TYPE+GETCC_FILL,
                      L0,
                      (char) i);

    twrtc(tape_name, D1);
  }

  /* wait for a moment before checking the queue length              */
  dlayc();

  /* get queue length                                                */
  status = tdspc_q(tape_name, D1);
  if (status == NULL)
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 10 Failed - Could not get tape status");
    exit(0);
  }

  /* sync tape                                                       */
  waitc();
  tsync(tape_name);
  if (waitc())
  {
    trsvc(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 10 Failed - Error synchronizing tape");
    exit(0);
  }

  /* release tape                                                    */
  trsvc(tape_name);

  /* rewind tape                                                     */
  if (tape_cntl(tape_name, CNTL_REW, NO_FALLBACK))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 10 Failed - Could not rewind tape");
    exit(0);
  }

  /* check the records written                                       */
  for (i = 0; i < 50; i++)
  {
    if (!(data_area = tape_read(tape_name, D1, L0)))
    {
      tape_close(tape_name);

      printmsg(full_msg, prefixed, 100, 'E',
               "OTEST1 Test 10 Failed - Could not read from tape");
      exit(0);
    }

    if ((data_area[0] != i) || (data_area[L0SIZE-1] != i)) {
      tape_close(tape_name);

      printmsg(full_msg, prefixed, 100, 'E',
               "OTEST1 Test 10 Failed - Data does not match");
      exit(0);
    }

    relcc(D1);
  }

  /*******************************************************************/
  /* Finished                                                        */
  /*******************************************************************/
  tape_close(tape_name);

  printmsg(full_msg, prefixed, 101, 'I',
           "OTEST1 Tests Succeeded", tape_name);
}


void tape_test_blk(char *tape_name)
{
  char *data_area = NULL;
  int i;
  struct tpqstat *status;

#define L4SIZE 4095

  /*******************************************************************/
  /* Test Case 1:  Open the tape                                     */
  /*******************************************************************/
  /* open the tape for r/w                                           */
  tape_open(tape_name, OUTPUT);

  /*******************************************************************/
  /* Test Case 2:  Initialize the tape                               */
  /*******************************************************************/
  /* make sure the tape is rewound                                   */
  if (tape_cntl(tape_name, CNTL_REW, NO_FALLBACK))
  {
    tape_close(tape_name);
    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 2 Failed - Could not rewind tape");
    exit(0);
  }

  /* write the sample data to tape                                   */
  for (i = 0; i < 60; i++)
  {
    data_area = getcc(D1,
                      GETCC_TYPE+GETCC_FILL,
                      L4,
                      (char) i);

    tape_write(tape_name, D1);
  }

  /* flush the tape buffer */
  tape_cntl(tape_name, CNTL_FLUSH);

  /*******************************************************************/
  /* Test Case 3:  Check the first record                            */
  /*******************************************************************/
  /* rewind the tape                                                 */
  if (tape_cntl(tape_name, CNTL_REW, NO_FALLBACK))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 3 Failed - Could not rewind tape");
    exit(0);
  }

  /* read the first record                                           */
  if ((data_area = tape_read(tape_name, D1, L4)) == NULL)
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 3 Failed - Could not read from tape");
    exit(0);
  }

  /* check a few spots                                               */
  if ((data_area[0] != 0) || (data_area[L4SIZE-1] != 0))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 3 Failed - Read did not match original");
    exit(0);
  }

  relcc(D1);

  /* read the second record                                          */
  if ((data_area = tape_read(tape_name, D1, L4)) == NULL)
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 3 Failed - Could not read from tape");
    exit(0);
  }

  /* check a few spots                                               */
  if ((data_area[0] != 1) || (data_area[L4SIZE-1] != 1))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 3 Failed - Data does not match");
    exit(0);
  }

  relcc(D1);

  /*******************************************************************/
  /* Test Case 4:  Move forward one record                           */
  /*******************************************************************/
  /* move forward one physical block (head at 7, so move to 14)      */
  if (tape_cntl(tape_name, CNTL_FSB, D1, 1))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 4 Failed - Could not seek tape forward");
    exit(0);
  }

  /* read the record                                                 */
  if (!(data_area = tape_read(tape_name, D1, L4)))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
               "OTEST1 Test 4 Failed - Could not read from tape");
    exit(0);
  }

  /* check a few spots                                               */
  if ((data_area[0] != 14) || (data_area[L4SIZE-1] != 14))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 4 Failed - Data does not match");
    exit(0);
  }

  relcc(D1);

  /*******************************************************************/
  /* Test Case 5:  Seek tape back 1 block                            */
  /*******************************************************************/
  /* move backward one block (head at 21, so move to 14)             */
  if (tape_cntl(tape_name, CNTL_BSB, D1, 1))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 5 Failed - Could not seek tape backward");
    exit(0);
  }

  /* read the record                                                 */
  if (!(data_area = tape_read(tape_name, D1, L4)))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 5 Failed - Could not read from tape");
    exit(0);
  }

  /* check a few spots                                               */
  if ((data_area[0] != 14) || (data_area[L4SIZE-1] != 14))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 5 Failed - Data does not match");
    exit(0);
  }

  relcc(D1);

  /*******************************************************************/
  /* Test Case 6:  Seek tape forward 2 blocks                        */
  /*******************************************************************/
  /* rewind the tape                                                 */
  if (tape_cntl(tape_name, CNTL_REW, NO_FALLBACK))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 6 Failed - Could not rewind tape");
    exit(0);
  }

  /* forward 2 blocks (at 0, so move to 14)                          */
  if (tape_cntl(tape_name, CNTL_FSB, D1, 2))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 6 Failed - Could not seek tape forward");
    exit(0);
  }

  /* read the record                                                 */
  if (!(data_area = tape_read(tape_name, D1, L4)))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 6 Failed - Could not read from tape");
    exit(0);
  }

  /* check a few spots                                               */
  if ((data_area[0] != 14) || (data_area[L4SIZE-1] != 14))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 6 Failed - Data does not match");
    exit(0);
  }

  relcc(D1);

  /*******************************************************************/
  /* Test Case 7:  Seek tape back 3 blocks                           */
  /*******************************************************************/

  /* backward 3 blocks (head at 21, so move to 0)                    */
  if (tape_cntl(tape_name, CNTL_BSB, D1, 3))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 7 Failed - Could not seek tape backward");
    exit(0);
  }

  /* read the record                                                 */
  if (!(data_area = tape_read(tape_name, D1, L4)))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 7 Failed - Could not read from tape");
    exit(0);
  }

  /* check a few spots                                               */
  if ((data_area[0] != 0) || (data_area[L4SIZE-1] != 0))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 7 Failed - Data does not match");
    exit(0);
  }

  relcc(D1);

  /*******************************************************************/
  /* Test Case 8:  Seek past end of tape                             */
  /*******************************************************************/

  /* rewind tape                                                     */
  if (tape_cntl(tape_name, CNTL_REW, NO_FALLBACK))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 8 Failed - Could not rewind tape");
    exit(0);
  }

  /* write one record                                                */
  data_area = getcc(D1,
                    GETCC_TYPE+GETCC_FILL,
                    L1,
                    '0');

  tape_write(tape_name, D1);
  tape_cntl(tape_name, CNTL_FLUSH);

  /* read past end of tape                                           */
  if ((data_area = tape_read(tape_name, D1, L1)))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 8 Failed - Read past end of tape");
    exit(0);
  }

  /* seek past end of tape                                           */
  if (!tape_cntl(tape_name, CNTL_FSB, D1, 2))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 8 Failed - Seeked past end of tape");
    exit(0);
  }

  /*******************************************************************/
  /* Test Case 9:  Seek past beginning of tape                       */
  /*******************************************************************/

  /* rewind                                                          */
  if (tape_cntl(tape_name, CNTL_REW, NO_FALLBACK))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 9 Failed - Could not rewind tape");
    exit(0);
  }

  /* seek backward                                                   */
  if (!tape_cntl(tape_name, CNTL_BSB, D1, 2))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 9 Failed - Seeked past beginning of tape");
    exit(0);
  }

  /*******************************************************************/
  /* Test Case 10:  Tape queue length and sync                       */
  /*******************************************************************/
  /* rewind tape                                                     */
  if (tape_cntl(tape_name, CNTL_REW, NO_FALLBACK))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 10 Failed - Could not rewind tape");
    exit(0);
  }

  /* grab tape */
  tasnc(tape_name);

  /* write lots of records */
  for (i = 0; i < 200; i++)
  {
    data_area = getcc(D1,
                      GETCC_TYPE+GETCC_FILL,
                      L4,
                      (char) i);

    twrtc(tape_name, D1);
  }

  dlayc();

  /* get queue length */
  status = tdspc_q(tape_name, D1);
  if (status == NULL)
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 10 Failed - Could not get tape status");
    exit(0);
  }

  /* sync tape */
  waitc();
  tsync(tape_name);
  if (waitc())
  {
    trsvc(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 10 Failed - Error synchronizing tape");
    exit(0);
  }

  /* release tape */
  trsvc(tape_name);

  /* rewind tape */
  if (tape_cntl(tape_name, CNTL_REW, NO_FALLBACK))
  {
    tape_close(tape_name);

    printmsg(full_msg, prefixed, 100, 'E',
             "OTEST1 Test 10 Failed - Could not rewind tape");
    exit(0);
  }

  /* check the records written */
  for (i = 0; i < 20; i++)
  {
    if (!(data_area = tape_read(tape_name, D1, L4)))
    {
      tape_close(tape_name);

      printmsg(full_msg, prefixed, 100, 'E',
               "OTEST1 Test 10 Failed - Could not read from tape");
      exit(0);
    }

    if ((data_area[0] != i) || (data_area[L4SIZE-1] != i)) {
      tape_close(tape_name);

      printmsg(full_msg, prefixed, 100, 'E',
               "OTEST1 Test 10 Failed - Data does not match");
      exit(0);
    }

    relcc(D1);
  }

  /*******************************************************************/
  /* Finished                                                        */
  /*******************************************************************/
  tape_close(tape_name);

  printmsg(full_msg, prefixed, 101, 'I',
           "OTEST1 Tests Succeeded", tape_name);
}

void tape_test()
{
  char *tape_name = "GEN";

  struct tpvstat *vstat = NULL;
  struct tpsec1 *base = NULL;
  struct tpsec1 *stat = NULL;

  /* determine the type of tape mounted (blocked/nonblocked          */
  vstat = tdspc_v(tape_name, ACTIVE, D1);

  /* check if the GEN tape is mounted or not */
  if (vstat == 0)                                                               //D14416
  {                                                                             //D14416
     printmsg(full_msg, prefixed, 102, 'E',                                     //D14416
           "OTEST1 Requires a mounted %s output tape", tape_name);              //D14416
     exit(0);                                                                   //D14416
  }                                                                             //D14416

  base = cinfc_fast(CINFC_CMMTS1);
  stat = base + vstat->modnum;

  if (stat->cpmti3.ct3blk)
  {
    printmsg(full_msg, prefixed, 103, 'I',
             "OTEST1 Testing a blocked tape");
    tape_test_blk(tape_name);
  }
  else
  {
    printmsg(full_msg, prefixed, 103, 'I',
             "OTEST1 Testing a non-blocked tape");
    tape_test_noblk(tape_name);
  }
}
